(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ngx-smooth-parallax', ['exports', '@angular/core', 'rxjs', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global["ngx-smooth-parallax"] = {}, global.ng.core, global.rxjs, global.ng.common));
})(this, (function (exports, i0, rxjs, common) { 'use strict';

    function _interopNamespace(e) {
        if (e && e.__esModule) return e;
        var n = Object.create(null);
        if (e) {
            Object.keys(e).forEach(function (k) {
                if (k !== 'default') {
                    var d = Object.getOwnPropertyDescriptor(e, k);
                    Object.defineProperty(n, k, d.get ? d : {
                        enumerable: true,
                        get: function () { return e[k]; }
                    });
                }
            });
        }
        n["default"] = e;
        return Object.freeze(n);
    }

    var i0__namespace = /*#__PURE__*/_interopNamespace(i0);

    var ParallaxComponent = /** @class */ (function () {
        function ParallaxComponent(zone, elementRef) {
            this.zone = zone;
            this.elementRef = elementRef;
            this.startOffsetParallax = 0;
            this.scrollVelocity = 7;
            this.onDestroy$ = new rxjs.Subject();
            this.observer = null;
            this.isNodePlatform = common.isPlatformServer(i0.PLATFORM_ID);
            this.parallaxObserver = new rxjs.BehaviorSubject(true);
            this.inertia = 2000;
            this.requestFrame = 0;
            this.thresholdSet = [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1];
        }
        Object.defineProperty(ParallaxComponent.prototype, "parallax", {
            set: function (value) {
                this.parallaxObserver.next(value);
            },
            enumerable: false,
            configurable: true
        });
        ParallaxComponent.prototype.ngAfterViewInit = function () {
            var _this = this;
            if (this.isNodePlatform) {
                return;
            }
            this.zone.runOutsideAngular(function () {
                _this.parallaxObserver.subscribe(function (enableParallax) {
                    if (enableParallax) {
                        _this.startParallax();
                    }
                    else {
                        _this.stopParallax();
                    }
                });
            });
        };
        ParallaxComponent.prototype.ngOnDestroy = function () {
            var _a;
            this.stopParallax();
            this.onDestroy$.next();
            (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect();
            this.parallaxObserver.complete();
        };
        ParallaxComponent.prototype.startParallax = function () {
            var _this = this;
            var _a, _b;
            (_b = (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect) === null || _b === void 0 ? void 0 : _b.call(_a);
            this.observer = new IntersectionObserver(function () { return _this.updateAnimation(); }, {
                root: null,
                rootMargin: 0 - this.startOffsetParallax + "px 0px 0px 0px",
                threshold: this.thresholdSet,
            });
            this.observer.observe(this.elementRef.nativeElement);
        };
        ParallaxComponent.prototype.stopParallax = function () {
            var _a, _b;
            if (this.isNodePlatform) {
                return;
            }
            cancelAnimationFrame(this.requestFrame);
            (_b = (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect) === null || _b === void 0 ? void 0 : _b.call(_a);
            this.updateTransform(0);
        };
        ParallaxComponent.prototype.updateAnimation = function () {
            cancelAnimationFrame(this.requestFrame);
            this.animate(new Date().getTime());
        };
        ParallaxComponent.prototype.animate = function (startTime) {
            var _this = this;
            var time = new Date().getTime();
            if (time - startTime > this.inertia) {
                return;
            }
            this.requestFrame = requestAnimationFrame(function () {
                _this.updateTransform(_this.getIntersectionRatio());
                _this.animate(startTime);
            });
        };
        ParallaxComponent.prototype.getIntersectionRatio = function () {
            var top = this.elementRef.nativeElement.getBoundingClientRect().top - this.startOffsetParallax;
            if (top >= 0) {
                return 0;
            }
            ;
            return Math.min(-top * (this.scrollVelocity / 10));
        };
        ParallaxComponent.prototype.updateTransform = function (translateY) {
            this.wrapper.nativeElement.style.transform = "translateY(" + translateY + "px)";
        };
        return ParallaxComponent;
    }());
    ParallaxComponent.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0__namespace, type: ParallaxComponent, deps: [{ token: i0__namespace.NgZone }, { token: i0__namespace.ElementRef }], target: i0__namespace.ɵɵFactoryTarget.Component });
    ParallaxComponent.ɵcmp = i0__namespace.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.1.5", type: ParallaxComponent, selector: "ngx-smooth-parallax", inputs: { startOffsetParallax: "startOffsetParallax", parallax: "parallax", scrollVelocity: "scrollVelocity" }, viewQueries: [{ propertyName: "wrapper", first: true, predicate: ["wrapper"], descendants: true }], ngImport: i0__namespace, template: "<div #wrapper>\n  <ng-content></ng-content>\n</div>", changeDetection: i0__namespace.ChangeDetectionStrategy.OnPush });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0__namespace, type: ParallaxComponent, decorators: [{
                type: i0.Component,
                args: [{
                        selector: 'ngx-smooth-parallax',
                        templateUrl: './parallax.component.html',
                        changeDetection: i0.ChangeDetectionStrategy.OnPush,
                    }]
            }], ctorParameters: function () { return [{ type: i0__namespace.NgZone }, { type: i0__namespace.ElementRef }]; }, propDecorators: { startOffsetParallax: [{
                    type: i0.Input
                }], parallax: [{
                    type: i0.Input
                }], scrollVelocity: [{
                    type: i0.Input
                }], wrapper: [{
                    type: i0.ViewChild,
                    args: ['wrapper']
                }] } });

    var NgxSmoothParallaxModule = /** @class */ (function () {
        function NgxSmoothParallaxModule() {
        }
        return NgxSmoothParallaxModule;
    }());
    NgxSmoothParallaxModule.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0__namespace, type: NgxSmoothParallaxModule, deps: [], target: i0__namespace.ɵɵFactoryTarget.NgModule });
    NgxSmoothParallaxModule.ɵmod = i0__namespace.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0__namespace, type: NgxSmoothParallaxModule, declarations: [ParallaxComponent], exports: [ParallaxComponent] });
    NgxSmoothParallaxModule.ɵinj = i0__namespace.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0__namespace, type: NgxSmoothParallaxModule });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0__namespace, type: NgxSmoothParallaxModule, decorators: [{
                type: i0.NgModule,
                args: [{
                        declarations: [
                            ParallaxComponent
                        ],
                        exports: [
                            ParallaxComponent
                        ]
                    }]
            }] });

    /*
     * Public API Surface of ngx-smooth-parallax
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NgxSmoothParallaxModule = NgxSmoothParallaxModule;
    exports.ParallaxComponent = ParallaxComponent;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=ngx-smooth-parallax.umd.js.map
