import * as i0 from '@angular/core';
import { PLATFORM_ID, Component, ChangeDetectionStrategy, Input, ViewChild, NgModule } from '@angular/core';
import { Subject, BehaviorSubject } from 'rxjs';
import { isPlatformServer } from '@angular/common';

class ParallaxComponent {
    constructor(zone, elementRef) {
        this.zone = zone;
        this.elementRef = elementRef;
        this.startOffsetParallax = 0;
        this.scrollVelocity = 7;
        this.onDestroy$ = new Subject();
        this.observer = null;
        this.isNodePlatform = isPlatformServer(PLATFORM_ID);
        this.parallaxObserver = new BehaviorSubject(true);
        this.inertia = 2000;
        this.requestFrame = 0;
        this.thresholdSet = [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1];
    }
    set parallax(value) {
        this.parallaxObserver.next(value);
    }
    ngAfterViewInit() {
        if (this.isNodePlatform) {
            return;
        }
        this.zone.runOutsideAngular(() => {
            this.parallaxObserver.subscribe((enableParallax) => {
                if (enableParallax) {
                    this.startParallax();
                }
                else {
                    this.stopParallax();
                }
            });
        });
    }
    ngOnDestroy() {
        var _a;
        this.stopParallax();
        this.onDestroy$.next();
        (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect();
        this.parallaxObserver.complete();
    }
    startParallax() {
        var _a, _b;
        (_b = (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect) === null || _b === void 0 ? void 0 : _b.call(_a);
        this.observer = new IntersectionObserver(() => this.updateAnimation(), {
            root: null,
            rootMargin: `${0 - this.startOffsetParallax}px 0px 0px 0px`,
            threshold: this.thresholdSet,
        });
        this.observer.observe(this.elementRef.nativeElement);
    }
    stopParallax() {
        var _a, _b;
        if (this.isNodePlatform) {
            return;
        }
        cancelAnimationFrame(this.requestFrame);
        (_b = (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect) === null || _b === void 0 ? void 0 : _b.call(_a);
        this.updateTransform(0);
    }
    updateAnimation() {
        cancelAnimationFrame(this.requestFrame);
        this.animate(new Date().getTime());
    }
    animate(startTime) {
        const time = new Date().getTime();
        if (time - startTime > this.inertia) {
            return;
        }
        this.requestFrame = requestAnimationFrame(() => {
            this.updateTransform(this.getIntersectionRatio());
            this.animate(startTime);
        });
    }
    getIntersectionRatio() {
        const top = this.elementRef.nativeElement.getBoundingClientRect().top - this.startOffsetParallax;
        if (top >= 0) {
            return 0;
        }
        ;
        return Math.min(-top * (this.scrollVelocity / 10));
    }
    updateTransform(translateY) {
        this.wrapper.nativeElement.style.transform = `translateY(${translateY}px)`;
    }
}
ParallaxComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0, type: ParallaxComponent, deps: [{ token: i0.NgZone }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component });
ParallaxComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.1.5", type: ParallaxComponent, selector: "ngx-smooth-parallax", inputs: { startOffsetParallax: "startOffsetParallax", parallax: "parallax", scrollVelocity: "scrollVelocity" }, viewQueries: [{ propertyName: "wrapper", first: true, predicate: ["wrapper"], descendants: true }], ngImport: i0, template: "<div #wrapper>\n  <ng-content></ng-content>\n</div>", changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0, type: ParallaxComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngx-smooth-parallax',
                    templateUrl: './parallax.component.html',
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: function () { return [{ type: i0.NgZone }, { type: i0.ElementRef }]; }, propDecorators: { startOffsetParallax: [{
                type: Input
            }], parallax: [{
                type: Input
            }], scrollVelocity: [{
                type: Input
            }], wrapper: [{
                type: ViewChild,
                args: ['wrapper']
            }] } });

class NgxSmoothParallaxModule {
}
NgxSmoothParallaxModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0, type: NgxSmoothParallaxModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxSmoothParallaxModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0, type: NgxSmoothParallaxModule, declarations: [ParallaxComponent], exports: [ParallaxComponent] });
NgxSmoothParallaxModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0, type: NgxSmoothParallaxModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.5", ngImport: i0, type: NgxSmoothParallaxModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        ParallaxComponent
                    ],
                    exports: [
                        ParallaxComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of ngx-smooth-parallax
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxSmoothParallaxModule, ParallaxComponent };
//# sourceMappingURL=ngx-smooth-parallax.js.map
